@@ -33,6 +33,7 @@ char test_file1[MAX_PATH_LEN + MAX_FILE_NAME_LEN];
 char test_file2[MAX_PATH_LEN + MAX_FILE_NAME_LEN];
 char test_unexisting[MAX_PATH_LEN + MAX_FILE_NAME_LEN];
 char test_dir[MAX_PATH_LEN + MAX_FILE_NAME_LEN];
+const char *test_vo;
 
 char *source_file = "file:///etc/group";
 char *test_spacedescriptor = "srm_test_space";
@@ -55,7 +56,7 @@ int MkDir(char *directory);
 void CopyFile(char *file)
 {
 	char *command;
-	asprintf (&command, "lcg-cp --nobdii -D srmv2 --vo dteam  %s %s ", source_file,file);
+	asprintf (&command, "lcg-cp --nobdii -D srmv2 --vo %s  %s %s ", test_vo, source_file,file);
 	//printf("%s \n",command);
 	system(command);
 }
@@ -77,9 +78,7 @@ int FakeCopy(char *surl) // does prepare to put and put done without copying a s
 	struct srm_preparetoput_output output_put;
 	SRM_LONG64 filesizes[1] ={ 0 };
 
-	context.verbose = 1;
-	context.errbufsz = 0;
-	context.srm_endpoint = test_srm_endpoint;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);
 	context.timeout = 3600;
 	context.version = VERSION_2_2;
 
@@ -146,9 +145,7 @@ int MkDir(char *directory)
 	asprintf(&test_surl_rmdir,"%s/test_dir",test_dir);
 	asprintf(&test_surl_mkdir,"%s/test_dir/1/2",test_dir);
 
-	context.verbose = 1;
-	context.errbufsz = 0;
-	context.srm_endpoint = test_srm_endpoint;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);
 	context.timeout = 3600;
 	context.version = VERSION_2_2;
 
@@ -180,9 +177,7 @@ START_TEST (test_directory_functions)
 	asprintf(&test_surl_rmdir,"%s/test_dir",test_dir);
 	asprintf(&test_surl_mkdir,"%s/test_dir/1/2",test_dir);
 
-	context.verbose = 1;
-	context.errbufsz = 0;
-	context.srm_endpoint = test_srm_endpoint;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);
 	context.timeout = 3600;
 	context.version = VERSION_2_2;
 
@@ -265,9 +260,7 @@ START_TEST (test_data_transfer_functions)
 	struct srmv2_pinfilestatus *filestatuses;
 	SRM_LONG64 filesizes[1] ={ 1024 };
 
-	context.verbose = 0;
-	context.errbufsz = 0;
-	context.srm_endpoint = test_srm_endpoint;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 0);
 	context.timeout = 3600;
 	context.version = VERSION_2_2;
 
@@ -349,13 +342,9 @@ START_TEST (test_srm_ping)
 	struct srm_context context;
 	int result;
 
-	context.verbose = 0;
-	context.errbuf = NULL;
-	context.errbufsz = 0;
 	context.version = VERSION_2_2;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);;
 
-
-	context.srm_endpoint = test_srm_endpoint;
 	result = srm_ping(&context,&output);
 	fail_if ((result  != 0),
 					"Expected Success !");
@@ -396,11 +385,8 @@ START_TEST (test_srm_space_management)
 	char *test_spacetoken_descriptor = test_spacedescriptor;
 	char *best_spacetoken;
 
-	context.verbose = 0;
-	context.errbuf = NULL;
-	context.errbufsz = 0;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);
 	context.version = VERSION_2_2;
-	context.srm_endpoint =  test_srm_endpoint;
 
 	input_reserve.desired_lifetime = 100;
 	input_reserve.desired_size = 1048576*2; // 2MB
@@ -489,11 +475,8 @@ START_TEST (test_srm_permissions)
 
 	int result;
 
-	context.verbose = 0;
-	context.errbuf = NULL;
-	context.errbufsz = 0;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);
 	context.version = VERSION_2_2;
-	context.srm_endpoint = test_srm_endpoint;
 
 	input.nbfiles = 1;
 	input.surls =  surls;
@@ -632,9 +615,14 @@ void SetUp()
     sprintf(test_file2, "%s/test_file2", test_dir);
     sprintf(test_unexisting, "%s/unexisting", test_dir);
 
+    test_vo = getenv("VO");
+    if (!test_vo)
+        test_vo = "dteam";
+
     printf("\nTest setup:\n\n");
     printf("SRM host: %s\n", srm_host);
-    printf("Test path: %s\n\n", test_dir);
+    printf("Test path: %s\n", test_dir);
+    printf("VO Name: %s\n\n", test_vo);
 }
 
 ///////////////////////////////////////////////
@@ -671,9 +659,7 @@ int TestLs(char *surl)
 	struct srm_ls_input input_ls;
 	struct srm_ls_output output_ls;
 
-	context.verbose = 1;
-	context.errbufsz = 0;
-	context.srm_endpoint = test_srm_endpoint;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);
 	context.timeout = 3600;
 	context.version = VERSION_2_2;
 
@@ -710,9 +696,7 @@ int TestPutDone(char** surls,char *token)
 	struct srm_putdone_input input_putdone;
 	struct srm_context context;
 
-	context.verbose = 1;
-	context.errbufsz = 0;
-	context.srm_endpoint = test_srm_endpoint;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);
 	context.timeout = 3600;
 	context.version = VERSION_2_2;
 
@@ -732,9 +716,7 @@ int TestPurgeFromSpace(char** surls,char *token)
 	struct srm_purgefromspace_output output_purge;
 	struct srm_context context;
 
-	context.verbose = 1;
-	context.errbufsz = 0;
-	context.srm_endpoint = test_srm_endpoint;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);
 	context.timeout = 3600;
 	context.version = VERSION_2_2;
 
@@ -754,9 +736,7 @@ int TestAbortRequest(char *token)
 	int c;
 	struct srm_context context;
 
-	context.verbose = 1;
-	context.errbufsz = 0;
-	context.srm_endpoint = test_srm_endpoint;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);
 	context.timeout = 3600;
 	context.version = VERSION_2_2;
 
@@ -773,9 +753,7 @@ int TestAbortFiles(char **files,char *token)
 	struct srm_context context;
 	struct srm_abort_files_input input;
 
-	context.verbose = 1;
-	context.errbufsz = 0;
-	context.srm_endpoint = test_srm_endpoint;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);
 	context.timeout = 3600;
 	context.version = VERSION_2_2;
 
@@ -795,9 +773,7 @@ int TestReleaseFiles(char **files,char *token)
 	struct srm_context context;
 	struct srm_releasefiles_input input;
 
-	context.verbose = 1;
-	context.errbufsz = 0;
-	context.srm_endpoint = test_srm_endpoint;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);
 	context.timeout = 3600;
 	context.version = VERSION_2_2;
 
@@ -817,9 +793,7 @@ int TestBringOnline(char **files,char **protocols)
 	struct srm_bringonline_output output_bringonline;
 	int a;
 
-	context.verbose = 1;
-	context.errbufsz = 0;
-	context.srm_endpoint = test_srm_endpoint;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);
 	context.timeout = 3600;
 	context.version = VERSION_2_2;
 
@@ -852,10 +826,7 @@ void TestSpaceTokensSpaceMetadata()
 	struct srm_getspacetokens_output output_space_tokens;
 
 
-
-	context.verbose = 1;
-	context.errbufsz = 0;
-	context.srm_endpoint = test_srm_endpoint;
+	srm_context_init(&context, test_srm_endpoint, NULL, 0, 1);
 	context.timeout = 3600;
 	context.version = VERSION_2_2;
 
@@ -882,9 +853,7 @@ int TestPing(char *endpoint)
 	struct srm_context context;
 	struct srm_ping_output output;
 
-	context.verbose = 1;
-	context.errbufsz = 0;
-	context.srm_endpoint = endpoint;
+	srm_context_init(&context, endpoint, NULL, 0, 1);
 	context.timeout = 3600;
 	context.version = VERSION_2_2;
 
