@@ -72,7 +72,7 @@ const char *srmv2_errmsg[] = {
 	"SRM_CUSTOM_STATUS"
 };
 static int srm_timeout_connect = 60;
-static int srm_timeout_sendreceive = 180;
+static int srm_timeout_sendreceive = 3600;
 
 int srm_get_timeout_connect ()
 {
@@ -97,40 +97,40 @@ void srm_set_timeout_sendreceive (int value)
 		srm_timeout_sendreceive = value;
 }
 
-void srm_context_init(struct srm_context *context,char *srm_endpoint,char *errbuf,int errbufsz,int verbose)
-{
-    GFAL_SRM_IFCE_ASSERT(context);
-	context->errbuf = errbuf;
-	context->errbufsz = errbufsz;
-	context->version = VERSION_2_2;
-	context->srm_endpoint = srm_endpoint;
-	context->timeout  = 60;
-	context->verbose = verbose;
-    context->timeout_conn = srm_get_timeout_connect ();
-    context->timeout_ops = srm_get_timeout_sendreceive();
-}
+
 
 void back_off_logic_init(struct srm_context *context,struct srm_internal_context *internal_context)
 {
     GFAL_SRM_IFCE_ASSERT(context);
     GFAL_SRM_IFCE_ASSERT(internal_context);
+    memset(internal_context,0, sizeof(struct srm_internal_context));
 
+    clock_gettime(CLOCK_MONOTONIC, &internal_context->end_time_spec);
     if (context->timeout > 0)
 	{
-		internal_context->end_time = (time(NULL) + context->timeout);
-	}
+        internal_context->relative_timeout = context->timeout;
+        internal_context->end_time = (time(NULL) + internal_context->relative_timeout);
+        internal_context->end_time_spec.tv_sec+= internal_context->relative_timeout;
+    }else{
+        internal_context->relative_timeout = 180;
+        internal_context->end_time = (time(NULL) + internal_context->relative_timeout);
+        internal_context->end_time_spec.tv_sec+= internal_context->relative_timeout;
+    }
 	internal_context->estimated_wait_time = -1;
 	internal_context->attempt = 1;
+
+    if(context->ext){
+        timespec_add(&(internal_context->current_waittime_spec), &(context->ext->min_waittime), &(internal_context->current_waittime_spec));
+    }
+
+    internal_context->random_seed = (unsigned int) time(NULL);
 }
-void set_estimated_wait_time(struct srm_internal_context *internal_context, int *time)
+
+
+void set_estimated_wait_time(struct srm_internal_context *internal_context, int my_time)
 {
-	if (time == NULL)
-	{
-		internal_context->estimated_wait_time = -1;
-	}else
-	{
-		internal_context->estimated_wait_time = *time;
-	}
+    // safety check on the serve value, if > relative_timeout -> fallback on eponential backoff logic
+    internal_context->estimated_wait_time = (my_time > 0 && my_time < internal_context->relative_timeout)?my_time:-1;
 }
 
 void srm_soap_init(struct soap *soap)
@@ -160,12 +160,24 @@ struct soap * srm_soap_init_context_new(struct srm_context* c){
     #ifdef GFAL_SECURE
     int flags;
     #endif
-    struct soap *soap_handle = soap_new();
-    soap_init (soap_handle);
+    struct soap *soap_handle = NULL;
+
+    if (c && c->ext && c->ext->keep_alive) {
+        soap_handle = soap_new2(SOAP_IO_KEEPALIVE, SOAP_IO_KEEPALIVE);
+        soap_handle->bind_flags |= SO_REUSEADDR;
+        soap_handle->max_keep_alive = 50;
+        soap_handle->accept_timeout = 0;
+	soap_handle->tcp_keep_alive = 1;
+        soap_handle->socket_flags = MSG_NOSIGNAL;
+    }
+    else {
+        soap_handle = soap_new();
+    }
+   
     soap_handle->namespaces = namespaces_srmv2;
 
     #ifdef GFAL_SECURE
-    flags = CGSI_OPT_DISABLE_NAME_CHECK;
+    flags = CGSI_OPT_DISABLE_NAME_CHECK | CGSI_OPT_KEEP_ALIVE;
     soap_register_plugin_arg (soap_handle, client_cgsi_plugin, &flags);
     #endif
 
@@ -330,6 +342,7 @@ int srm_print_error_status_additional(struct srm_context *context,struct srm2__T
 	}
 	return statuscode2errno (status->statusCode);
 }
+
 void srm_print_explanation(char **explanation,struct srm2__TReturnStatus *reqstatp,const char *srmfunc)
 {
 	if (reqstatp->explanation != NULL && reqstatp->explanation[0])
@@ -341,34 +354,62 @@ void srm_print_explanation(char **explanation,struct srm2__TReturnStatus *reqsta
 				srmfunc, statuscode2errmsg(
 						reqstatp->statusCode));
 }
+
+
+int wait_for_new_attempt_min_max_ng(srm_context_t context, struct srm_internal_context *internal_context){
+    int ret =-1; // default -> go timeout
+    struct timespec sleep_time, current_time, end_sleep_time;
+    clock_gettime(CLOCK_MONOTONIC, &current_time);
+    timespec_add(&(internal_context->current_waittime_spec), &(current_time), &(end_sleep_time));
+    if( timespec_cmp(&end_sleep_time, &(internal_context->end_time_spec), <) ){
+        //printf(" I wil sleep %ld %ld", internal_context->current_waittime_spec.tv_sec, internal_context->current_waittime_spec.tv_nsec);
+        while(timespec_cmp(&(current_time), &end_sleep_time, <)){ // go to sleep for the exact required time
+            call_function.call_usleep(1);
+            clock_gettime(CLOCK_MONOTONIC, &current_time);
+        }
+
+        ret =0;
+    }
+
+     // double the wait time if < max
+    timespec_add(&(internal_context->current_waittime_spec),&(internal_context->current_waittime_spec),
+                 &(sleep_time));
+    if(timespec_cmp(&sleep_time, &(context->ext->max_waittime), <) ){
+        timespec_copy(&(internal_context->current_waittime_spec), &sleep_time);
+    }else{
+         timespec_copy(&(internal_context->current_waittime_spec), &(context->ext->max_waittime));
+    }
+    return ret;
+}
+
+
 // Returns -1 for timeout
 // Returns  0 for wait finished
 int wait_for_new_attempt(struct srm_internal_context *internal_context)  // Or Timeout
 {
-	const int last_chance_sec_before_end = 2; // 2 seconds before the end
-	int random_limit;
-	int random_wait;
-	int wait_till_end=0;
+    const time_t last_chance_sec_before_end = 2; // 2 seconds before the end
+    int random_limit;
+    int random_wait;
+    time_t wait_till_end=0;
 	time_t after_sleep;
 	after_sleep = time(NULL) ;
-	srand( time(NULL) ); // new seed
 
-	if (internal_context->estimated_wait_time <= -1)
+    if (internal_context->estimated_wait_time <= 0 && internal_context->estimated_wait_time < ( internal_context->end_time - time(NULL)))
 	{
 		// Exponential logic
 		if (internal_context->attempt<=10) // Maximum 10 attempts
 		{
-			random_limit = (1<<(internal_context->attempt));
-			random_wait = (rand() % random_limit);
+            random_limit = (time_t) (1<<(internal_context->attempt));
+            random_wait = (rand_r(&(internal_context->random_seed)) % random_limit)+1;
 			internal_context->attempt++;
-			after_sleep += random_wait;
+            after_sleep += (time_t) random_wait;
 			if (after_sleep >= internal_context->end_time)
 			{
-				wait_till_end = internal_context->end_time - time(NULL) -last_chance_sec_before_end; //try the last hope before the end
+                wait_till_end = internal_context->end_time - time(NULL) -  last_chance_sec_before_end; //try the last hope before the end
 				if (wait_till_end>0) 
 					call_function.call_sleep(wait_till_end); 
-				else // deadline outdated, also..
-					return -1; 					// simply return in timeout
+                else // deadline outdated,
+                    return -1; 					// simply return in timeout
 			}else
 			{
 				call_function.call_sleep(random_wait);
@@ -378,33 +419,42 @@ int wait_for_new_attempt(struct srm_internal_context *internal_context)  // Or T
 			// Timeout, attempts exceeded
 			return -1;
 		}
-	}else
-	if (internal_context->estimated_wait_time == 0)
-	{
-		// Timeout
-		return -1;
-	}else
-	{
-		after_sleep += internal_context->estimated_wait_time ;
-		if (after_sleep >= internal_context->end_time)
-		{
-			wait_till_end = internal_context->end_time - (time(NULL)+last_chance_sec_before_end);
-			if (wait_till_end>0)
-			{
-				call_function.call_sleep(wait_till_end);
-			}else
-			{
-				// Timeout
-				return -1;
-			}
-		}else
-		{
-			call_function.call_sleep(internal_context->estimated_wait_time );
-		}
-	}
+    }else{
+            after_sleep += internal_context->estimated_wait_time ;
+            if (after_sleep >= internal_context->end_time)
+            {
+                wait_till_end = internal_context->end_time - (time(NULL)+last_chance_sec_before_end);
+                if (wait_till_end>0)
+                {
+                    call_function.call_sleep(wait_till_end);
+                }else
+                {
+                    // Timeout
+                    return -1;
+                }
+            }else
+            {
+                call_function.call_sleep(internal_context->estimated_wait_time );
+            }
+    }
 
 	return 0;
 }
+
+int wait_switch_auto(srm_context_t context, struct srm_internal_context *internal_context){
+    if(context->ext == NULL)
+        return wait_for_new_attempt(internal_context);
+    switch(context->ext->polling_logic){
+        case SRM_POLLING_LOGIC_OLD:
+            return wait_for_new_attempt(internal_context);
+        case SRM_POLLING_LOGIC_MIN_MAX_EXP:
+        default:
+            return wait_for_new_attempt_min_max_ng(context, internal_context);
+
+    }
+    return -1;
+}
+
 // Return all statuses timeout, failure,internal error, queued
 srm_call_status back_off_logic(struct srm_context *context,const char *srmfunc,
 		struct srm_internal_context *internal_context,struct srm2__TReturnStatus  *retstatus)
@@ -422,7 +472,7 @@ srm_call_status back_off_logic(struct srm_context *context,const char *srmfunc,
 		switch (retstatus->statusCode )
 		{
 			case SRM_USCOREINTERNAL_USCOREERROR:
-				if (wait_for_new_attempt(internal_context) != 0)
+                if (wait_switch_auto(context,internal_context)  != 0)
 				{
 					srm_errmsg (context, "[%s][%s][ETIMEDOUT] %s: User timeout over",
 							err_msg_begin,srmfunc, context->srm_endpoint);
@@ -433,7 +483,7 @@ srm_call_status back_off_logic(struct srm_context *context,const char *srmfunc,
 				return srm_call_status_INTERNAL_ERROR;
 			case SRM_USCOREREQUEST_USCOREQUEUED:
 			case SRM_USCOREREQUEST_USCOREINPROGRESS:
-				if (wait_for_new_attempt(internal_context) != 0)
+                if (wait_switch_auto(context,internal_context) != 0)
 				{
 					srm_errmsg (context, "[%s][%s][ETIMEDOUT] %s: User timeout over",
 							err_msg_begin,srmfunc, context->srm_endpoint);
@@ -996,25 +1046,26 @@ int copy_mdfilestatuses(struct srm2__TReturnStatus *reqstatp,
 		if (repfs->pathDetailArray[i]->status)
 		{
 			(*statuses)[i].status = statuscode2errno(repfs->pathDetailArray[i]->status->statusCode);
-		}
-		if ((*statuses)[i].status) {
-			if (repfs->pathDetailArray[i]->status->explanation && repfs->pathDetailArray[i]->status->explanation[0])
-			{
-                (void) asprintf (&((*statuses)[i].explanation), "[%s][%s][%s] %s",err_msg_begin,
-						srmfunc, statuscode2errmsg(repfs->pathDetailArray[i]->status->statusCode),
-						repfs->pathDetailArray[i]->status->explanation);
-			}else if (reqstatp->explanation != NULL && reqstatp->explanation[0] && strncasecmp (reqstatp->explanation, "failed for all", 14))
-			{
-                 (void) asprintf (&((*statuses)[i].explanation), "[%s][%s][%s] %s",err_msg_begin,
-						srmfunc, statuscode2errmsg(repfs->pathDetailArray[i]->status->statusCode),
-						reqstatp->explanation);
-			}else
-			{
-                 (void) asprintf (&((*statuses)[i].explanation), "[%s][%s][%s] <none>",err_msg_begin,
-						srmfunc, statuscode2errmsg(repfs->pathDetailArray[i]->status->statusCode));
-			}
 
-			continue;
+			if ((*statuses)[i].status) {
+				if (repfs->pathDetailArray[i]->status->explanation && repfs->pathDetailArray[i]->status->explanation[0])
+				{
+					(void) asprintf (&((*statuses)[i].explanation), "[%s][%s][%s] %s",err_msg_begin,
+							srmfunc, statuscode2errmsg(repfs->pathDetailArray[i]->status->statusCode),
+							repfs->pathDetailArray[i]->status->explanation);
+				}else if (reqstatp->explanation != NULL && reqstatp->explanation[0] && strncasecmp (reqstatp->explanation, "failed for all", 14))
+				{
+					 (void) asprintf (&((*statuses)[i].explanation), "[%s][%s][%s] %s",err_msg_begin,
+							srmfunc, statuscode2errmsg(repfs->pathDetailArray[i]->status->statusCode),
+							reqstatp->explanation);
+				}else
+				{
+					 (void) asprintf (&((*statuses)[i].explanation), "[%s][%s][%s] <none>",err_msg_begin,
+							srmfunc, statuscode2errmsg(repfs->pathDetailArray[i]->status->statusCode));
+				}
+
+				continue;
+			}
 		}
 
 		if (repfs->pathDetailArray[i]->size)
