@@ -19,9 +19,9 @@
 #include <errno.h>
 #include <regex.h>
 #include <assert.h>
+#include "srm_ifce_internal.h"
 #include "srm_soap.h"
-#include "srm_util.h"
-#include "srm_dependencies.h"
+
 
 // Utility functions
 int srmv2_check_srm_root(const char* surl);
@@ -169,6 +169,10 @@ int srmv2_ls_async_internal(struct srm_context *context,
 				}
 			}
 			break;
+        case srm_call_status_TIMEOUT:
+            errno = ETIMEDOUT;
+            ret = -1;
+            break;
 		default:
 			errno = srm_call_err(context,output->retstatus,srmfunc);
 			ret = -1;
@@ -257,6 +261,10 @@ int srmv2_status_of_ls_request_async_internal(struct srm_context *context,
 			break;
 		case srm_call_status_QUEUED:
 			break;
+        case srm_call_status_TIMEOUT:
+            errno = ETIMEDOUT;
+            ret = -1;
+            break;
 		default:
 			errno = srm_call_err(context,output->retstatus,srmfunc);
 			ret = -1;
@@ -318,6 +326,12 @@ int srmv2_rm(struct srm_context *context,struct srm_rm_input *input,struct srm_r
 
 	}while (internal_context.current_status == srm_call_status_INTERNAL_ERROR);
 
+    if(internal_context.current_status == srm_call_status_TIMEOUT){
+               errno = ETIMEDOUT;
+               srm_soap_free(soap);
+               return (-1);
+     }
+
 	repfs = rep.srmRmResponse->arrayOfFileStatuses;
 
 	if (!repfs || repfs->__sizestatusArray < 1 || !repfs->statusArray) {
@@ -354,7 +368,6 @@ int srmv2_rm(struct srm_context *context,struct srm_rm_input *input,struct srm_r
 //struct srmv2_filestatus **statuses, char *errbuf, int errbufsz, int timeout)
 int srmv2_rmdir(struct srm_context *context,struct srm_rmdir_input *input,struct srm_rmdir_output *output)
 {
-	int flags;
 	int ret;
 	struct srm2__srmRmdirResponse_ rep;
 	struct srm2__srmRmdirRequest req;
@@ -395,6 +408,13 @@ int srmv2_rmdir(struct srm_context *context,struct srm_rmdir_input *input,struct
 
 	}while (internal_context.current_status == srm_call_status_INTERNAL_ERROR);
 
+
+    if(internal_context.current_status == srm_call_status_TIMEOUT){
+               errno = ETIMEDOUT;
+               srm_soap_free(soap);
+               return (-1);
+     }
+
 	if ((output->statuses = (struct srmv2_filestatus*) calloc (1, sizeof (struct srmv2_filestatus))) == NULL) {
 		errno = ENOMEM;
 		srm_soap_free(soap);
@@ -416,7 +436,6 @@ int srmv2_mkdir(struct srm_context *context,struct srm_mkdir_input *input)
 {
 	char* file = NULL;
     int ret = -1;
-	int flags;
 	int sav_errno = 0;
 	char *p, *endp;
 	struct srm2__srmMkdirResponse_ rep;
@@ -508,9 +527,7 @@ int srmv2_extend_file_lifetime(struct srm_context *context,
 	struct srm2__ArrayOfTSURLLifetimeReturnStatus *repfs;
 	struct srm2__srmExtendFileLifeTimeResponse_ rep;
 	struct srm2__srmExtendFileLifeTimeRequest req;
-	struct srm2__TReturnStatus *reqstatp;
     struct soap* soap = srm_soap_init_context_new(context);
-	int i = 0;
 	const char srmfunc[] = "ExtendFileLifeTime";
 
 	
@@ -573,8 +590,8 @@ int srmv2_check_srm_root(const char* surl)
 {
     int ret = 0;
     static regex_t re;
-    static is_compiled = 0;
-    static const char* regexp = "^srm\://[^/]*/$";
+    static int is_compiled = 0;
+    static const char* regexp = "^srm://[^/]*/$";
     #define SRMV1_CHECK_SRM_ROOT_NMATCH 1
     regmatch_t match[SRMV1_CHECK_SRM_ROOT_NMATCH];
 
